@@ -50,7 +50,6 @@ import com.cloud.user.Account;
         requestHasSensitiveInfo = false, responseHasSensitiveInfo = false)
 public class CreateNetworkOfferingCmd extends BaseCmd {
     public static final Logger s_logger = Logger.getLogger(CreateNetworkOfferingCmd.class.getName());
-    private static final String s_name = "createnetworkofferingresponse";
 
     /////////////////////////////////////////////////////
     //////////////// API parameters /////////////////////
@@ -127,6 +126,11 @@ public class CreateNetworkOfferingCmd extends BaseCmd {
             description = "true if network offering is meant to be used for VPC, false otherwise.")
     private Boolean forVpc;
 
+    @Parameter(name = ApiConstants.FOR_TUNGSTEN,
+            type = CommandType.BOOLEAN,
+            description = "true if network offering is meant to be used for Tungsten-Fabric, false otherwise.")
+    private Boolean forTungsten;
+
     @Parameter(name = ApiConstants.DETAILS, type = CommandType.MAP, since = "4.2.0", description = "Network offering details in key/value pairs."
             + " Supported keys are internallbprovider/publiclbprovider with service provider as a value, and"
             + " promiscuousmode/macaddresschanges/forgedtransmits with true/false as value to accept/reject the security settings if available for a nic/portgroup")
@@ -201,10 +205,6 @@ public class CreateNetworkOfferingCmd extends BaseCmd {
         return networkRate;
     }
 
-    public static String getName() {
-        return s_name;
-    }
-
     public Long getServiceOfferingId() {
         return serviceOfferingId;
     }
@@ -240,6 +240,10 @@ public class CreateNetworkOfferingCmd extends BaseCmd {
         return forVpc;
     }
 
+    public Boolean getForTungsten() {
+        return forTungsten;
+    }
+
     public Boolean getEgressDefaultPolicy() {
         if (egressDefaultPolicy == null) {
             return true;
@@ -362,11 +366,6 @@ public class CreateNetworkOfferingCmd extends BaseCmd {
     /////////////////////////////////////////////////////
     /////////////// API Implementation///////////////////
     /////////////////////////////////////////////////////
-    @Override
-    public String getCommandName() {
-        return s_name;
-    }
-
     @Override
     public long getEntityOwnerId() {
         return Account.ACCOUNT_ID_SYSTEM;
