@@ -39,7 +39,6 @@ import com.cloud.utils.Pair;
 public class ListSnapshotsCmd extends BaseListTaggedResourcesCmd {
     public static final Logger s_logger = Logger.getLogger(ListSnapshotsCmd.class.getName());
 
-    private static final String s_name = "listsnapshotsresponse";
 
     /////////////////////////////////////////////////////
     //////////////// API parameters /////////////////////
@@ -98,11 +97,6 @@ public class ListSnapshotsCmd extends BaseListTaggedResourcesCmd {
     /////////////// API Implementation///////////////////
     /////////////////////////////////////////////////////
 
-    @Override
-    public String getCommandName() {
-        return s_name;
-    }
-
     @Override
     public ApiCommandResourceType getApiResourceType() {
         return ApiCommandResourceType.Snapshot;
