@@ -32,6 +32,8 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Random;
+import java.util.stream.Collectors;
 
 import org.apache.cloudstack.engine.subsystem.api.storage.StoragePoolAllocator;
 import org.apache.cloudstack.framework.config.ConfigKey;
@@ -46,6 +48,7 @@ import org.mockito.InjectMocks;
 import org.mockito.Mock;
 import org.mockito.Mockito;
 import org.mockito.Spy;
+import org.mockito.stubbing.Answer;
 import org.mockito.runners.MockitoJUnitRunner;
 
 import com.cloud.agent.AgentManager;
@@ -68,13 +71,18 @@ import com.cloud.service.ServiceOfferingVO;
 import com.cloud.service.dao.ServiceOfferingDao;
 import com.cloud.storage.DiskOfferingVO;
 import com.cloud.storage.ScopeType;
+import com.cloud.storage.Storage;
 import com.cloud.storage.StorageManager;
 import com.cloud.storage.StoragePool;
 import com.cloud.storage.StoragePoolHostVO;
+import com.cloud.storage.VMTemplateVO;
+import com.cloud.storage.VMTemplateZoneVO;
 import com.cloud.storage.Volume;
 import com.cloud.storage.VolumeVO;
 import com.cloud.storage.dao.DiskOfferingDao;
 import com.cloud.storage.dao.StoragePoolHostDao;
+import com.cloud.storage.dao.VMTemplateDao;
+import com.cloud.storage.dao.VMTemplateZoneDao;
 import com.cloud.storage.dao.VolumeDao;
 import com.cloud.utils.exception.CloudRuntimeException;
 import com.cloud.vm.VirtualMachine.State;
@@ -134,6 +142,10 @@ public class VirtualMachineManagerImplTest {
 
     @Mock
     private DiskOfferingDao diskOfferingDaoMock;
+    @Mock
+    VMTemplateDao templateDao;
+    @Mock
+    VMTemplateZoneDao templateZoneDao;
 
     @Mock
     private HostDao hostDaoMock;
@@ -365,9 +377,26 @@ public class VirtualMachineManagerImplTest {
         Mockito.verify(storagePoolVoMock, Mockito.times(0)).getId();
     }
 
+    @Test
+    public void allowVolumeMigrationsForPowerFlexStorage() {
+        Mockito.doReturn(true).when(storagePoolVoMock).isManaged();
+        Mockito.doReturn(Storage.StoragePoolType.PowerFlex).when(storagePoolVoMock).getPoolType();
+
+        virtualMachineManagerImpl.executeManagedStorageChecksWhenTargetStoragePoolProvided(storagePoolVoMock, volumeVoMock, Mockito.mock(StoragePoolVO.class));
+
+        Mockito.verify(storagePoolVoMock).isManaged();
+        Mockito.verify(storagePoolVoMock, Mockito.times(0)).getId();
+    }
+
     @Test
     public void executeManagedStorageChecksWhenTargetStoragePoolProvidedTestCurrentStoragePoolEqualsTargetPool() {
         Mockito.doReturn(true).when(storagePoolVoMock).isManaged();
+        // return any storage type except powerflex/scaleio
+        List<Storage.StoragePoolType> values = Arrays.asList(Storage.StoragePoolType.values());
+        when(storagePoolVoMock.getPoolType()).thenAnswer((Answer<Storage.StoragePoolType>) invocation -> {
+            List<Storage.StoragePoolType> filteredValues = values.stream().filter(v -> v != Storage.StoragePoolType.PowerFlex).collect(Collectors.toList());
+            int randomIndex = new Random().nextInt(filteredValues.size());
+            return filteredValues.get(randomIndex); });
 
         virtualMachineManagerImpl.executeManagedStorageChecksWhenTargetStoragePoolProvided(storagePoolVoMock, volumeVoMock, storagePoolVoMock);
 
@@ -378,6 +407,12 @@ public class VirtualMachineManagerImplTest {
     @Test(expected = CloudRuntimeException.class)
     public void executeManagedStorageChecksWhenTargetStoragePoolProvidedTestCurrentStoragePoolNotEqualsTargetPool() {
         Mockito.doReturn(true).when(storagePoolVoMock).isManaged();
+        // return any storage type except powerflex/scaleio
+        List<Storage.StoragePoolType> values = Arrays.asList(Storage.StoragePoolType.values());
+        when(storagePoolVoMock.getPoolType()).thenAnswer((Answer<Storage.StoragePoolType>) invocation -> {
+            List<Storage.StoragePoolType> filteredValues = values.stream().filter(v -> v != Storage.StoragePoolType.PowerFlex).collect(Collectors.toList());
+            int randomIndex = new Random().nextInt(filteredValues.size());
+            return filteredValues.get(randomIndex); });
 
         virtualMachineManagerImpl.executeManagedStorageChecksWhenTargetStoragePoolProvided(storagePoolVoMock, volumeVoMock, Mockito.mock(StoragePoolVO.class));
     }
@@ -776,4 +811,88 @@ public class VirtualMachineManagerImplTest {
         Mockito.doReturn(isOfferingUsingLocal).when(diskOfferingMock).isUseLocalStorage();
         virtualMachineManagerImpl.checkIfNewOfferingStorageScopeMatchesStoragePool(vmInstanceMock, diskOfferingMock);
     }
+
+    @Test
+    public void checkIfTemplateNeededForCreatingVmVolumesExistingRootVolumes() {
+        long vmId = 1L;
+        VMInstanceVO vm = Mockito.mock(VMInstanceVO.class);
+        Mockito.when(vm.getId()).thenReturn(vmId);
+        Mockito.when(volumeDaoMock.findReadyRootVolumesByInstance(vmId)).thenReturn(List.of(Mockito.mock(VolumeVO.class)));
+        virtualMachineManagerImpl.checkIfTemplateNeededForCreatingVmVolumes(vm);
+    }
+
+    @Test(expected = CloudRuntimeException.class)
+    public void checkIfTemplateNeededForCreatingVmVolumesMissingTemplate() {
+        long vmId = 1L;
+        long templateId = 1L;
+        VMInstanceVO vm = Mockito.mock(VMInstanceVO.class);
+        Mockito.when(vm.getId()).thenReturn(vmId);
+        Mockito.when(vm.getTemplateId()).thenReturn(templateId);
+        Mockito.when(volumeDaoMock.findReadyRootVolumesByInstance(vmId)).thenReturn(null);
+        Mockito.when(templateDao.findById(templateId)).thenReturn(null);
+        virtualMachineManagerImpl.checkIfTemplateNeededForCreatingVmVolumes(vm);
+    }
+
+    @Test(expected = CloudRuntimeException.class)
+    public void checkIfTemplateNeededForCreatingVmVolumesMissingZoneTemplate() {
+        long vmId = 1L;
+        long templateId = 1L;
+        long dcId = 1L;
+        VMInstanceVO vm = Mockito.mock(VMInstanceVO.class);
+        Mockito.when(vm.getId()).thenReturn(vmId);
+        Mockito.when(vm.getTemplateId()).thenReturn(templateId);
+        Mockito.when(vm.getDataCenterId()).thenReturn(dcId);
+        Mockito.when(volumeDaoMock.findReadyRootVolumesByInstance(vmId)).thenReturn(null);
+        VMTemplateVO template = Mockito.mock(VMTemplateVO.class);
+        Mockito.when(vm.getId()).thenReturn(templateId);
+        Mockito.when(templateDao.findById(templateId)).thenReturn(template);
+        virtualMachineManagerImpl.checkIfTemplateNeededForCreatingVmVolumes(vm);
+    }
+
+    @Test
+    public void checkIfTemplateNeededForCreatingVmVolumesTemplateAvailable() {
+        long vmId = 1L;
+        long templateId = 1L;
+        long dcId = 1L;
+        VMInstanceVO vm = Mockito.mock(VMInstanceVO.class);
+        Mockito.when(vm.getId()).thenReturn(vmId);
+        Mockito.when(vm.getTemplateId()).thenReturn(templateId);
+        Mockito.when(vm.getDataCenterId()).thenReturn(dcId);
+        Mockito.when(volumeDaoMock.findReadyRootVolumesByInstance(vmId)).thenReturn(new ArrayList<>());
+        VMTemplateVO template = Mockito.mock(VMTemplateVO.class);
+        Mockito.when(template.getId()).thenReturn(templateId);
+        Mockito.when(templateDao.findById(templateId)).thenReturn(template);
+        Mockito.when(templateZoneDao.findByZoneTemplate(dcId, templateId)).thenReturn(Mockito.mock(VMTemplateZoneVO.class));
+        virtualMachineManagerImpl.checkIfTemplateNeededForCreatingVmVolumes(vm);
+    }
+
+    @Test
+    public void checkAndAttemptMigrateVmAcrossClusterNonValid() {
+        // Below scenarios shouldn't result in VM migration
+
+        VMInstanceVO vm = Mockito.mock(VMInstanceVO.class);
+        Mockito.when(vm.getHypervisorType()).thenReturn(HypervisorType.KVM);
+        virtualMachineManagerImpl.checkAndAttemptMigrateVmAcrossCluster(vm, 1L, new HashMap<>());
+
+        Mockito.when(vm.getHypervisorType()).thenReturn(HypervisorType.VMware);
+        Mockito.when(vm.getLastHostId()).thenReturn(null);
+        virtualMachineManagerImpl.checkAndAttemptMigrateVmAcrossCluster(vm, 1L, new HashMap<>());
+
+        Long destinationClusterId = 10L;
+        Mockito.when(vm.getLastHostId()).thenReturn(1L);
+        HostVO hostVO = Mockito.mock(HostVO.class);
+        Mockito.when(hostVO.getClusterId()).thenReturn(destinationClusterId);
+        Mockito.when(hostDaoMock.findById(1L)).thenReturn(hostVO);
+        virtualMachineManagerImpl.checkAndAttemptMigrateVmAcrossCluster(vm, destinationClusterId, new HashMap<>());
+
+        destinationClusterId = 20L;
+        Map<Volume, StoragePool> map = new HashMap<>();
+        StoragePool pool1 = Mockito.mock(StoragePool.class);
+        Mockito.when(pool1.getClusterId()).thenReturn(10L);
+        map.put(Mockito.mock(Volume.class), pool1);
+        StoragePool pool2 = Mockito.mock(StoragePool.class);
+        Mockito.when(pool2.getClusterId()).thenReturn(null);
+        map.put(Mockito.mock(Volume.class), pool2);
+        virtualMachineManagerImpl.checkAndAttemptMigrateVmAcrossCluster(vm, destinationClusterId, map);
+    }
 }
