@@ -18,6 +18,7 @@
 //
 package com.cloud.hypervisor.kvm.resource.wrapper;
 
+import org.apache.cloudstack.direct.download.DirectDownloadHelper;
 import org.apache.cloudstack.agent.directdownload.CheckUrlAnswer;
 import org.apache.cloudstack.agent.directdownload.CheckUrlCommand;
 import org.apache.log4j.Logger;
@@ -25,8 +26,6 @@ import org.apache.log4j.Logger;
 import com.cloud.hypervisor.kvm.resource.LibvirtComputingResource;
 import com.cloud.resource.CommandWrapper;
 import com.cloud.resource.ResourceWrapper;
-import com.cloud.utils.UriUtils;
-import com.cloud.utils.storage.QCOW2Utils;
 
 @ResourceWrapper(handles =  CheckUrlCommand.class)
 public class LibvirtCheckUrlCommand extends CommandWrapper<CheckUrlCommand, CheckUrlAnswer, LibvirtComputingResource> {
@@ -37,21 +36,16 @@ public class LibvirtCheckUrlCommand extends CommandWrapper<CheckUrlCommand, Chec
     public CheckUrlAnswer execute(CheckUrlCommand cmd, LibvirtComputingResource serverResource) {
         final String url = cmd.getUrl();
         s_logger.info("Checking URL: " + url);
-        boolean checkResult = true;
         Long remoteSize = null;
-        try {
-            UriUtils.checkUrlExistence(url);
-
-            if ("qcow2".equalsIgnoreCase(cmd.getFormat())) {
-                remoteSize = QCOW2Utils.getVirtualSize(url);
-            } else {
-                remoteSize = UriUtils.getRemoteSize(url);
+        boolean checkResult = DirectDownloadHelper.checkUrlExistence(url, cmd.isFollowRedirects());
+        if (checkResult) {
+            remoteSize = DirectDownloadHelper.getFileSize(url, cmd.getFormat(), cmd.isFollowRedirects());
+            if (remoteSize == null || remoteSize < 0) {
+                s_logger.error(String.format("Couldn't properly retrieve the remote size of the template on " +
+                        "url %s, obtained size = %s", url, remoteSize));
+                return new CheckUrlAnswer(false, remoteSize);
             }
         }
-        catch (IllegalArgumentException e) {
-            s_logger.warn(e.getMessage());
-            checkResult = false;
-        }
         return new CheckUrlAnswer(checkResult, remoteSize);
     }
 }
