@@ -113,7 +113,8 @@ public class KVMStoragePoolManager {
                     s_logger.warn(String.format("Duplicate StorageAdaptor type %s, not loading %s", info.storagePoolType().toString(), storageAdaptor.getName()));
                 } else {
                     try {
-                        this._storageMapper.put(info.storagePoolType().toString(), storageAdaptor.newInstance());
+                        s_logger.info(String.format("adding storage adaptor for %s", storageAdaptor.getName()));
+                        this._storageMapper.put(info.storagePoolType().toString(), storageAdaptor.getDeclaredConstructor().newInstance());
                     } catch (Exception ex) {
                        throw new CloudRuntimeException(ex.toString());
                     }
