@@ -45,33 +45,39 @@ public interface KubernetesClusterService extends PluggableService, Configurable
             "cloud.kubernetes.cluster.network.offering",
             "DefaultNetworkOfferingforKubernetesService",
             "Name of the network offering that will be used to create isolated network in which Kubernetes cluster VMs will be launched",
-            false);
+            false,
+            KubernetesServiceEnabled.key());
     static final ConfigKey<Long> KubernetesClusterStartTimeout = new ConfigKey<Long>("Advanced", Long.class,
             "cloud.kubernetes.cluster.start.timeout",
             "3600",
             "Timeout interval (in seconds) in which start operation for a Kubernetes cluster should be completed",
-            true);
+            true,
+            KubernetesServiceEnabled.key());
     static final ConfigKey<Long> KubernetesClusterScaleTimeout = new ConfigKey<Long>("Advanced", Long.class,
             "cloud.kubernetes.cluster.scale.timeout",
             "3600",
             "Timeout interval (in seconds) in which scale operation for a Kubernetes cluster should be completed",
-            true);
+            true,
+            KubernetesServiceEnabled.key());
     static final ConfigKey<Long> KubernetesClusterUpgradeTimeout = new ConfigKey<Long>("Advanced", Long.class,
             "cloud.kubernetes.cluster.upgrade.timeout",
             "3600",
             "Timeout interval (in seconds) in which upgrade operation for a Kubernetes cluster should be completed. Not strictly obeyed while upgrade is in progress on a node",
-            true);
+            true,
+            KubernetesServiceEnabled.key());
     static final ConfigKey<Boolean> KubernetesClusterExperimentalFeaturesEnabled = new ConfigKey<Boolean>("Advanced", Boolean.class,
             "cloud.kubernetes.cluster.experimental.features.enabled",
             "false",
             "Indicates whether experimental feature for Kubernetes cluster such as Docker private registry are enabled or not",
-            true);
+            true,
+            KubernetesServiceEnabled.key());
     static final ConfigKey<Integer> KubernetesMaxClusterSize = new ConfigKey<Integer>("Advanced", Integer.class,
             "cloud.kubernetes.cluster.max.size",
             "10",
             "Maximum size of the kubernetes cluster.",
-            true, ConfigKey.Scope.Account);
-
+            true,
+            ConfigKey.Scope.Account,
+            KubernetesServiceEnabled.key());
 
     KubernetesCluster findById(final Long id);
 
