@@ -296,6 +296,7 @@ class CsDevice:
 
 
 class CsIP:
+    DEFAULT_MTU = '1500'
 
     def __init__(self, dev, config):
         self.dev = dev
@@ -320,6 +321,8 @@ class CsIP:
                 logging.info("Configuring address %s on device %s", self.ip(), self.dev)
                 cmd = "ip addr add dev %s %s brd +" % (self.dev, self.ip())
                 CsHelper.execute(cmd)
+                cmd = "ifconfig %s mtu %s"  % (self.dev, self.mtu())
+                CsHelper.execute(cmd)
             except Exception as e:
                 logging.info("Exception occurred ==> %s" % e)
 
@@ -390,6 +393,12 @@ class CsIP:
             return self.address['public_ip']
         return "unknown"
 
+    def mtu(self):
+        logging.info(self.address)
+        if "mtu" in self.address:
+            return self.address['mtu']
+        return CsIP.DEFAULT_MTU
+
     def setup_router_control(self):
         if self.config.is_vpc():
             return
@@ -423,13 +432,27 @@ class CsIP:
                             "-d %s/32 -j FIREWALL_%s" % (self.address['public_ip'], self.address['public_ip'])])
             self.fw.append(["mangle", "front",
                             "-A FIREWALL_%s " % self.address['public_ip'] +
-                            "-m state --state RELATED,ESTABLISHED -j ACCEPT"])
+                            "-m state --state RELATED,ESTABLISHED -j RETURN"])
             self.fw.append(["mangle", "",
                             "-A FIREWALL_%s -j DROP" % self.address['public_ip']])
             self.fw.append(["mangle", "",
                             "-I VPN_%s -m state --state RELATED,ESTABLISHED -j ACCEPT" % self.address['public_ip']])
             self.fw.append(["mangle", "",
                             "-A VPN_%s -j RETURN" % self.address['public_ip']])
+            self.fw.append(
+                ["", "front", "-A FORWARD -j NETWORK_STATS_%s" % self.dev])
+            self.fw.append(
+                ["", "front", "-A INPUT -j NETWORK_STATS_%s" % self.dev])
+            self.fw.append(
+                ["", "front", "-A OUTPUT -j NETWORK_STATS_%s" % self.dev])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s -i eth0 -o %s" % (self.dev, self.dev)])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s -i %s -o eth0" % (self.dev, self.dev)])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s -o %s ! -i eth0 -p tcp" % (self.dev, self.dev)])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s -i %s ! -o eth0 -p tcp" % (self.dev, self.dev)])
             self.fw.append(["nat", "",
                             "-A POSTROUTING -o %s -j SNAT --to-source %s" % (self.dev, self.cl.get_eth2_ip())])
             self.fw.append(["mangle", "",
@@ -556,18 +579,17 @@ class CsIP:
             self.fw.append(
                 ["", "front", "-A FORWARD -j NETWORK_STATS_%s" % self.dev])
             self.fw.append(
-                ["", "front", "-A NETWORK_STATS_%s -s %s -o %s" % (self.dev, self.cl.get_vpccidr(), self.dev)])
+                ["", "front", "-A INPUT -j NETWORK_STATS_%s" % self.dev])
             self.fw.append(
-                ["", "front", "-A NETWORK_STATS_%s -d %s -i %s" % (self.dev, self.cl.get_vpccidr(), self.dev)])
-
-        self.fw.append(["", "front", "-A FORWARD -j NETWORK_STATS"])
-        self.fw.append(["", "front", "-A INPUT -j NETWORK_STATS"])
-        self.fw.append(["", "front", "-A OUTPUT -j NETWORK_STATS"])
-
-        self.fw.append(["", "", "-A NETWORK_STATS -i eth0 -o eth2 -p tcp"])
-        self.fw.append(["", "", "-A NETWORK_STATS -i eth2 -o eth0 -p tcp"])
-        self.fw.append(["", "", "-A NETWORK_STATS ! -i eth0 -o eth2 -p tcp"])
-        self.fw.append(["", "", "-A NETWORK_STATS -i eth2 ! -o eth0 -p tcp"])
+                ["", "front", "-A OUTPUT -j NETWORK_STATS_%s" % self.dev])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s -s %s -o %s" % (self.dev, self.cl.get_vpccidr(), self.dev)])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s -d %s -i %s" % (self.dev, self.cl.get_vpccidr(), self.dev)])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s ! -s %s -o %s -p tcp" % (self.dev, self.cl.get_vpccidr(), self.dev)])
+            self.fw.append(
+                ["", "", "-A NETWORK_STATS_%s ! -d %s -i %s -p tcp" % (self.dev, self.cl.get_vpccidr(), self.dev)])
 
         self.fw.append(["filter", "", "-A INPUT -d 224.0.0.18/32 -j ACCEPT"])
         self.fw.append(["filter", "", "-A INPUT -d 225.0.0.50/32 -j ACCEPT"])
@@ -610,6 +632,9 @@ class CsIP:
                             if "nw_type" in address and address["nw_type"] == "guest":
                                 route.add_network_route(self.dev, str(address["network"]))
 
+                if self.get_type() in ["public"]:
+                    CsRule(self.dev).addRule("from " + str(self.address["network"]))
+
                 route.add_network_route(self.dev, str(self.address["network"]))
 
             CsHelper.execute("sudo ip route flush cache")
