@@ -32,6 +32,11 @@ setup_console_proxy() {
   public_ip=`getPublicIp`
   echo "$public_ip $NAME" >> /etc/hosts
 
+  log_it "Applying iptables rule for VNC port ${VNCPORT}"
+  sed -i 's/8080/${VNCPORT}/' /etc/iptables/rules.v4
+  echo "${VNCPORT}" > /root/vncport
+  log_it "Creating VNC port ${VNCPORT} file for VNC server configuration"
+
   disable_rpfilter
   enable_fwding 0
   enable_irqbalance 0
