@@ -14,7 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-""" Tests for Kubernetes supported version """
+""" Tests for Kubernetes cluster """
 
 #Import Local Modules
 from marvin.cloudstackTestCase import cloudstackTestCase
@@ -37,11 +37,16 @@ from marvin.cloudstackAPI import (listInfrastructure,
 from marvin.cloudstackException import CloudstackAPIException
 from marvin.codes import PASS, FAILED
 from marvin.lib.base import (Template,
+                             NetworkOffering,
                              Network,
                              ServiceOffering,
                              Account,
                              StoragePool,
-                             Configurations)
+                             Configurations,
+                             VpcOffering,
+                             VPC,
+                             NetworkACLList,
+                             NetworkACL)
 from marvin.lib.utils import (cleanup_resources,
                               validateList,
                               random_gen)
@@ -57,6 +62,11 @@ import time, io, yaml
 _multiprocess_shared_ = True
 
 k8s_cluster = None
+VPC_DATA = {
+    "cidr": "10.1.0.0/22",
+    "tier1_gateway": "10.1.1.1",
+    "tier_netmask": "255.255.255.0"
+}
 
 class TestKubernetesCluster(cloudstackTestCase):
 
@@ -75,6 +85,10 @@ class TestKubernetesCluster(cloudstackTestCase):
         cls.setup_failed = False
         cls._cleanup = []
         cls.kubernetes_version_ids = []
+        cls.vpcAllowAllAclDetailsMap = {}
+
+        cls.k8s_version_from = cls.services["cks_kubernetes_version_upgrade_from"]
+        cls.k8s_version_to = cls.services["cks_kubernetes_version_upgrade_to"]
 
         if cls.hypervisorNotSupported == False:
             cls.endpoint_url = Configurations.list(cls.apiclient, name="endpoint.url")[0].value
@@ -94,20 +108,20 @@ class TestKubernetesCluster(cloudstackTestCase):
 
             if cls.setup_failed == False:
                 try:
-                    cls.kubernetes_version_1_23_3 = cls.addKubernetesSupportedVersion(cls.services["cks_kubernetes_versions"]["1.23.3"])
-                    cls.kubernetes_version_ids.append(cls.kubernetes_version_1_23_3.id)
+                    cls.kubernetes_version_v1 = cls.addKubernetesSupportedVersion(cls.services["cks_kubernetes_versions"][cls.k8s_version_from])
+                    cls.kubernetes_version_ids.append(cls.kubernetes_version_v1.id)
                 except Exception as e:
                     cls.setup_failed = True
                     cls.debug("Failed to get Kubernetes version ISO in ready state, version=%s, url=%s, %s" %
-                        (cls.services["cks_kubernetes_versions"]["1.23.3"]["semanticversion"], cls.services["cks_kubernetes_versions"]["1.23.3"]["url"], e))
+                        (cls.services["cks_kubernetes_versions"][cls.k8s_version_from]["semanticversion"], cls.services["cks_kubernetes_versions"][cls.k8s_version_from]["url"], e))
             if cls.setup_failed == False:
                 try:
-                    cls.kubernetes_version_1_24_0 = cls.addKubernetesSupportedVersion(cls.services["cks_kubernetes_versions"]["1.24.0"])
-                    cls.kubernetes_version_ids.append(cls.kubernetes_version_1_24_0.id)
+                    cls.kubernetes_version_v2 = cls.addKubernetesSupportedVersion(cls.services["cks_kubernetes_versions"][cls.k8s_version_to])
+                    cls.kubernetes_version_ids.append(cls.kubernetes_version_v2.id)
                 except Exception as e:
                     cls.setup_failed = True
                     cls.debug("Failed to get Kubernetes version ISO in ready state, version=%s, url=%s, %s" %
-                        (cls.services["cks_kubernetes_versions"]["1.24.0"]["semanticversion"], cls.services["cks_kubernetes_versions"]["1.24.0"]["url"], e))
+                        (cls.services["cks_kubernetes_versions"][cls.k8s_version_to]["semanticversion"], cls.services["cks_kubernetes_versions"][cls.k8s_version_to]["url"], e))
 
             if cls.setup_failed == False:
                 cks_offering_data = cls.services["cks_service_offering"]
@@ -342,11 +356,7 @@ class TestKubernetesCluster(cloudstackTestCase):
         return
 
     def tearDown(self):
-        try:
-            cleanup_resources(self.apiclient, self.cleanup)
-        except Exception as e:
-            raise Exception("Warning: Exception during cleanup : %s" % e)
-        return
+        super(TestKubernetesCluster, self).tearDown()
 
     @attr(tags=["advanced", "smoke"], required_hardware="true")
     @skipTestIf("hypervisorNotSupported")
@@ -359,20 +369,20 @@ class TestKubernetesCluster(cloudstackTestCase):
         if self.setup_failed == True:
             self.fail("Setup incomplete")
         global k8s_cluster
-        k8s_cluster = self.getValidKubernetesCluster(version=self.kubernetes_version_1_24_0)
+        k8s_cluster = self.getValidKubernetesCluster(version=self.kubernetes_version_v2)
 
         self.debug("Downgrading Kubernetes cluster with ID: %s to a lower version. This should fail!" % k8s_cluster.id)
 
         try:
-            k8s_cluster = self.upgradeKubernetesCluster(k8s_cluster.id, self.kubernetes_version_1_23_3.id)
-            self.debug("Invalid CKS Kubernetes HA cluster deployed with ID: %s. Deleting it and failing test." % self.kubernetes_version_1_23_3.id)
+            k8s_cluster = self.upgradeKubernetesCluster(k8s_cluster.id, self.kubernetes_version_v1.id)
+            self.debug("Invalid CKS Kubernetes HA cluster deployed with ID: %s. Deleting it and failing test." % self.kubernetes_version_v1.id)
             self.deleteKubernetesClusterAndVerify(k8s_cluster.id, False, True)
             self.fail("Kubernetes cluster downgrade to a lower Kubernetes supported version. Must be an error.")
         except Exception as e:
             self.debug("Upgrading Kubernetes cluster with invalid Kubernetes supported version check successful, API failure: %s" % e)
             self.deleteKubernetesClusterAndVerify(k8s_cluster.id, False, True)
 
-        self.verifyKubernetesClusterUpgrade(k8s_cluster, self.kubernetes_version_1_24_0.id)
+        self.verifyKubernetesClusterUpgrade(k8s_cluster, self.kubernetes_version_v2.id)
         return
 
     @attr(tags=["advanced", "smoke"], required_hardware="true")
@@ -386,17 +396,17 @@ class TestKubernetesCluster(cloudstackTestCase):
         if self.setup_failed == True:
             self.fail("Setup incomplete")
         global k8s_cluster
-        k8s_cluster = self.getValidKubernetesCluster(version=self.kubernetes_version_1_23_3)
+        k8s_cluster = self.getValidKubernetesCluster(version=self.kubernetes_version_v1)
 
         time.sleep(self.services["sleep"])
         self.debug("Upgrading Kubernetes cluster with ID: %s" % k8s_cluster.id)
         try:
-            k8s_cluster = self.upgradeKubernetesCluster(k8s_cluster.id, self.kubernetes_version_1_24_0.id)
+            k8s_cluster = self.upgradeKubernetesCluster(k8s_cluster.id, self.kubernetes_version_v2.id)
         except Exception as e:
             self.deleteKubernetesClusterAndVerify(k8s_cluster.id, False, True)
             self.fail("Failed to upgrade Kubernetes cluster due to: %s" % e)
 
-        self.verifyKubernetesClusterUpgrade(k8s_cluster, self.kubernetes_version_1_24_0.id)
+        self.verifyKubernetesClusterUpgrade(k8s_cluster, self.kubernetes_version_v2.id)
         return
 
     @attr(tags=["advanced", "smoke"], required_hardware="true")
@@ -444,12 +454,12 @@ class TestKubernetesCluster(cloudstackTestCase):
         if self.setup_failed == True:
             self.fail("Setup incomplete")
         global k8s_cluster
-        k8s_cluster = self.getValidKubernetesCluster(version=self.kubernetes_version_1_24_0)
+        k8s_cluster = self.getValidKubernetesCluster(version=self.kubernetes_version_v2)
 
         self.debug("Autoscaling Kubernetes cluster with ID: %s" % k8s_cluster.id)
         try:
             k8s_cluster = self.autoscaleKubernetesCluster(k8s_cluster.id, 1, 2)
-            self.verifyKubernetesClusterAutocale(k8s_cluster, 1, 2)
+            self.verifyKubernetesClusterAutoscale(k8s_cluster, 1, 2)
 
             up = self.waitForAutoscalerPodInRunningState(k8s_cluster.id)
             self.assertTrue(up, "Autoscaler pod failed to run")
@@ -544,17 +554,17 @@ class TestKubernetesCluster(cloudstackTestCase):
         if self.default_network:
             self.skipTest("HA cluster on shared network requires external ip address, skipping it")
         global k8s_cluster
-        k8s_cluster = self.getValidKubernetesCluster(1, 2, version=self.kubernetes_version_1_23_3)
+        k8s_cluster = self.getValidKubernetesCluster(1, 2, version=self.kubernetes_version_v1)
         time.sleep(self.services["sleep"])
 
         self.debug("Upgrading HA Kubernetes cluster with ID: %s" % k8s_cluster.id)
         try:
-            k8s_cluster = self.upgradeKubernetesCluster(k8s_cluster.id, self.kubernetes_version_1_24_0.id)
+            k8s_cluster = self.upgradeKubernetesCluster(k8s_cluster.id, self.kubernetes_version_v2.id)
         except Exception as e:
             self.deleteKubernetesClusterAndVerify(k8s_cluster.id, False, True)
             self.fail("Failed to upgrade Kubernetes HA cluster due to: %s" % e)
 
-        self.verifyKubernetesClusterUpgrade(k8s_cluster, self.kubernetes_version_1_24_0.id)
+        self.verifyKubernetesClusterUpgrade(k8s_cluster, self.kubernetes_version_v2.id)
         self.debug("Kubernetes cluster with ID: %s successfully upgraded" % k8s_cluster.id)
         return
 
@@ -576,6 +586,33 @@ class TestKubernetesCluster(cloudstackTestCase):
         self.debug("Deleting Kubernetes cluster with ID: %s" % k8s_cluster.id)
         return
 
+    @attr(tags=["advanced", "smoke"], required_hardware="true")
+    @skipTestIf("hypervisorNotSupported")
+    def test_10_vpc_tier_kubernetes_cluster(self):
+        """Test to deploy a Kubernetes cluster on VPC
+
+        # Validate the following:
+        # 1. Deploy a Kubernetes cluster on a VPC tier
+        # 2. Destroy it
+        """
+        if self.setup_failed == True:
+            self.fail("Setup incomplete")
+        global k8s_cluster
+        if k8s_cluster != None and k8s_cluster.id != None:
+            self.deleteKubernetesClusterAndVerify(k8s_cluster.id, False, True)
+        self.createVpcOffering()
+        self.createVpcTierOffering()
+        self.deployVpc()
+        self.deployNetworkTier()
+        self.default_network = self.vpc_tier
+        k8s_cluster = self.getValidKubernetesCluster(1, 1)
+
+        self.debug("Deleting Kubernetes cluster with ID: %s" % k8s_cluster.id)
+        self.deleteKubernetesClusterAndVerify(k8s_cluster.id)
+        self.debug("Kubernetes cluster with ID: %s successfully deleted" % k8s_cluster.id)
+        k8s_cluster = None
+        return
+
     def createKubernetesCluster(self, name, version_id, size=1, control_nodes=1):
         createKubernetesClusterCmd = createKubernetesCluster.createKubernetesClusterCmd()
         createKubernetesClusterCmd.name = name
@@ -659,7 +696,7 @@ class TestKubernetesCluster(cloudstackTestCase):
         # Does a cluster already exist ?
         if cluster == None or cluster.id == None:
             if not version:
-                version = self.kubernetes_version_1_24_0
+                version = self.kubernetes_version_v2
             self.debug("No existing cluster available, k8s_cluster: %s" % cluster)
             return self.createNewKubernetesCluster(version, size, control_nodes)
 
@@ -669,7 +706,7 @@ class TestKubernetesCluster(cloudstackTestCase):
             # Check the version only if specified
             valid = valid and cluster.kubernetesversionid == version.id
         else:
-            version = self.kubernetes_version_1_24_0
+            version = self.kubernetes_version_v2
 
         if valid:
             cluster_id = cluster.id
@@ -783,7 +820,7 @@ class TestKubernetesCluster(cloudstackTestCase):
         self.verifyKubernetesClusterState(cluster_response, 'Running')
         self.verifyKubernetesClusterSize(cluster_response, size, control_nodes)
 
-    def verifyKubernetesClusterAutocale(self, cluster_response, minsize, maxsize):
+    def verifyKubernetesClusterAutoscale(self, cluster_response, minsize, maxsize):
         """Check if Kubernetes cluster state and node sizes are valid after upgrade"""
 
         self.verifyKubernetesClusterState(cluster_response, 'Running')
@@ -816,3 +853,91 @@ class TestKubernetesCluster(cloudstackTestCase):
             'Stopped',
             "KubernetesCluster not stopped in DB, {}".format(db_cluster_state)
         )
+
+    def createVpcOffering(self):
+        off_service = self.services["vpc_offering"]
+        self.vpc_offering = VpcOffering.create(
+            self.apiclient,
+            off_service
+        )
+        self.cleanup.append(self.vpc_offering)
+        self.vpc_offering.update(self.apiclient, state='Enabled')
+
+    def createVpcTierOffering(self):
+        off_service = self.services["nw_offering_isolated_vpc"]
+        self.vpc_tier_offering = NetworkOffering.create(
+            self.apiclient,
+            off_service,
+            conservemode=False
+        )
+        self.cleanup.append(self.vpc_tier_offering)
+        self.vpc_tier_offering.update(self.apiclient, state='Enabled')
+
+    def deployAllowEgressDenyIngressVpcInternal(self, cidr):
+        service = self.services["vpc"]
+        service["cidr"] = cidr
+        vpc = VPC.create(
+            self.apiclient,
+            service,
+            vpcofferingid=self.vpc_offering.id,
+            zoneid=self.zone.id,
+            account=self.account.name,
+            domainid=self.account.domainid
+        )
+        self.cleanup.append(vpc)
+        acl = NetworkACLList.create(
+            self.apiclient,
+            services={},
+            name="allowegressdenyingress",
+            description="allowegressdenyingress",
+            vpcid=vpc.id
+        )
+        rule ={
+            "protocol": "all",
+            "traffictype": "egress",
+        }
+        NetworkACL.create(self.apiclient,
+            services=rule,
+            aclid=acl.id
+        )
+        rule["traffictype"] = "ingress"
+        rule["action"] = "deny"
+        NetworkACL.create(self.apiclient,
+            services=rule,
+            aclid=acl.id
+        )
+        self.vpcAllowAllAclDetailsMap[vpc.id] = acl.id
+        return vpc
+
+    def deployVpc(self):
+        self.vpc = self.deployAllowEgressDenyIngressVpcInternal(VPC_DATA["cidr"])
+
+    def deployNetworkTierInternal(self, network_offering_id, vpc_id, tier_gateway, tier_netmask, acl_id=None, tier_name=None):
+        if not acl_id and vpc_id in self.vpcAllowAllAclDetailsMap:
+            acl_id = self.vpcAllowAllAclDetailsMap[vpc_id]
+        service = self.services["ntwk"]
+        if tier_name:
+            service["name"] = tier_name
+            service["displaytext"] = "vpc-%s" % tier_name
+        network = Network.create(
+            self.apiclient,
+            service,
+            self.account.name,
+            self.account.domainid,
+            networkofferingid=network_offering_id,
+            vpcid=vpc_id,
+            zoneid=self.zone.id,
+            gateway=tier_gateway,
+            netmask=tier_netmask,
+            aclid=acl_id
+        )
+        self.cleanup.append(network)
+        return network
+
+    def deployNetworkTier(self):
+        self.vpc_tier = self.deployNetworkTierInternal(
+            self.vpc_tier_offering.id,
+            self.vpc.id,
+            VPC_DATA["tier1_gateway"],
+            VPC_DATA["tier_netmask"]
+        )
