@@ -23,6 +23,7 @@ from marvin.lib.base import *
 from marvin.lib.common import *
 from marvin.lib.utils import (random_gen)
 from nose.plugins.attrib import attr
+from marvin.lib.decoratorGenerators import skipTestIf
 
 import time
 
@@ -42,6 +43,7 @@ class TestMetrics(cloudstackTestCase):
                                zoneid=cls.zone.id,
                                type='Routing')[0]
         cls.cluster = cls.apiclient.listClusters(listClusters.listClustersCmd())[0]
+        cls.mgtSvrDetails = cls.config.__dict__["mgtSvr"][0].__dict__
         cls._cleanup = []
         cls.disk_offering = DiskOffering.create(
             cls.apiclient,
@@ -52,12 +54,12 @@ class TestMetrics(cloudstackTestCase):
             cls.apiclient,
             cls.services["service_offering"]
         )
+        cls._cleanup.append(cls.service_offering)
         cls.template = get_test_template(
             cls.apiclient,
             cls.zone.id,
             cls.hypervisor
         )
-        cls._cleanup.append(cls.service_offering)
         cls.domain = get_domain(cls.apiclient)
         cls.account = Account.create(
             cls.apiclient,
@@ -66,11 +68,80 @@ class TestMetrics(cloudstackTestCase):
             domainid=cls.domain.id
         )
         cls._cleanup.append(cls.account)
+        cls.hypervisorNotSupported = True
+        if cls.hypervisor.lower() != 'simulator':
+            cls.hypervisorNotSupported = False
+            cls.vm_stats_interval_cfg = Configurations.list(cls.apiclient, name='vm.stats.interval')[0].value
+            cls.vm_stats_max_retention_time_cfg = Configurations.list(cls.apiclient, name='vm.stats.max.retention.time')[0].value
+            cls.vm_stats_user_vm_only_cfg = Configurations.list(cls.apiclient, name='vm.stats.user.vm.only')[0].value
+            cls.vm_disk_stats_interval_cfg = Configurations.list(cls.apiclient, name='vm.disk.stats.interval')[0].value
+            cls.vm_disk_stats_interval_min_cfg = Configurations.list(cls.apiclient, name='vm.disk.stats.interval.min')[0].value
+            cls.vm_disk_stats_max_retention_time_cfg = Configurations.list(cls.apiclient, name='vm.disk.stats.max.retention.time')[0].value
+            cls.vm_disk_stats_retention_enabled_cfg = Configurations.list(cls.apiclient, name='vm.disk.stats.retention.enabled')[0].value
+            Configurations.update(cls.apiclient, 'vm.stats.interval', value='60000')
+            Configurations.update(cls.apiclient, 'vm.stats.max.retention.time', value='7200')
+            Configurations.update(cls.apiclient, 'vm.stats.user.vm.only', value='false')
+            Configurations.update(cls.apiclient, 'vm.disk.stats.interval', value='60')
+            Configurations.update(cls.apiclient, 'vm.disk.stats.interval.min', value='60')
+            Configurations.update(cls.apiclient, 'vm.disk.stats.max.retention.time', value='7200')
+            Configurations.update(cls.apiclient, 'vm.disk.stats.retention.enabled', value='true')
+            cls.restartServer()
 
     @classmethod
     def tearDownClass(cls):
+        if cls.hypervisor.lower() != 'simulator':
+            cls.updateConfiguration('vm.stats.interval', cls.vm_stats_interval_cfg)
+            cls.updateConfiguration('vm.stats.max.retention.time', cls.vm_stats_max_retention_time_cfg)
+            cls.updateConfiguration('vm.stats.user.vm.only', cls.vm_stats_user_vm_only_cfg)
+            cls.updateConfiguration('vm.disk.stats.interval', cls.vm_disk_stats_interval_cfg)
+            cls.updateConfiguration('vm.disk.stats.interval.min', cls.vm_disk_stats_interval_min_cfg)
+            cls.updateConfiguration('vm.disk.stats.max.retention.time', cls.vm_disk_stats_max_retention_time_cfg)
+            cls.updateConfiguration('vm.disk.stats.retention.enabled', cls.vm_disk_stats_retention_enabled_cfg)
+            cls.restartServer()
         super(TestMetrics, cls).tearDownClass()
 
+    @classmethod
+    def restartServer(cls):
+        """Restart management server"""
+
+        cls.debug("Restarting management server")
+        sshClient = SshClient(
+                    cls.mgtSvrDetails["mgtSvrIp"],
+            22,
+            cls.mgtSvrDetails["user"],
+            cls.mgtSvrDetails["passwd"]
+        )
+        command = "service cloudstack-management stop"
+        sshClient.execute(command)
+
+        command = "service cloudstack-management start"
+        sshClient.execute(command)
+
+        #Waits for management to come up in 5 mins, when it's up it will continue
+        timeout = time.time() + 300
+        while time.time() < timeout:
+            if cls.isManagementUp() is True:
+                # allow hosts to be ready for deployment
+                time.sleep(30)
+                return
+            time.sleep(5)
+        cls.setup_failed = True
+        cls.debug("Management server did not come up, failing")
+        return
+
+    @classmethod
+    def isManagementUp(cls):
+        try:
+            cls.apiclient.listInfrastructure(listInfrastructure.listInfrastructureCmd())
+            return True
+        except Exception:
+            return False
+
+    @classmethod
+    def updateConfiguration(cls, config, value):
+        if value is not None:
+            Configurations.update(cls.apiclient, config, value=value)
+
     def setUp(self):
         self.userapiclient = self.testClient.getUserApiClient(
             UserName=self.account.name,
@@ -389,6 +460,111 @@ class TestMetrics(cloudstackTestCase):
 
         return
 
+    @attr(tags = ["advanced", "advancedns", "smoke", "basic"], required_hardware="true")
+    @skipTestIf("hypervisorNotSupported")
+    def test_list_vms_metrics_history(self):
+        #deploy VM
+        self.small_virtual_machine = VirtualMachine.create(
+                                        self.apiclient,
+                                        self.services["virtual_machine"],
+                                        serviceofferingid=self.service_offering.id,
+                                        templateid=self.template.id,
+                                        zoneid=self.zone.id
+                                        )
+        self.cleanup.append(self.small_virtual_machine)
+
+        # Wait for 2 minutes
+        time.sleep(120)
+
+        cmd = listVirtualMachinesUsageHistory.listVirtualMachinesUsageHistoryCmd()
+        cmd.id = self.small_virtual_machine.id
+
+        result = self.apiclient.listVirtualMachinesUsageHistory(cmd)[0]
+
+        self.assertEqual(result.id, self.small_virtual_machine.id)
+        self.assertTrue(hasattr(result, 'stats'))
+        self.assertTrue(type(result.stats) == list and len(result.stats) > 0)
+        self.validate_vm_stats(result.stats[0])
+
+        return
+
+    @attr(tags = ["advanced", "advancedns", "smoke", "basic"], required_hardware="true")
+    @skipTestIf("hypervisorNotSupported")
+    def test_list_system_vms_metrics_history(self):
+        cmd = listSystemVmsUsageHistory.listSystemVmsUsageHistoryCmd()
+        now = datetime.datetime.now() - datetime.timedelta(minutes=15)
+        start_time = now.strftime("%Y-%m-%d %H:%M:%S")
+        cmd.startdate = start_time
+
+        result = self.apiclient.listSystemVmsUsageHistory(cmd)[0]
+
+        self.assertTrue(hasattr(result, 'stats'))
+        self.assertTrue(type(result.stats) == list and len(result.stats) > 0)
+        self.validate_vm_stats(result.stats[0])
+
+        return
+
+    @attr(tags = ["advanced", "advancedns", "smoke", "basic"], required_hardware="true")
+    @skipTestIf("hypervisorNotSupported")
+    def test_list_volumes_metrics_history(self):
+        #deploy VM
+        self.small_virtual_machine = VirtualMachine.create(
+                                        self.apiclient,
+                                        self.services["virtual_machine"],
+                                        serviceofferingid=self.service_offering.id,
+                                        templateid=self.template.id,
+                                        zoneid=self.zone.id
+                                        )
+        self.cleanup.append(self.small_virtual_machine)
+
+        currentHost = Host.list(self.apiclient, id=self.small_virtual_machine.hostid)[0]
+        if currentHost.hypervisor.lower() == "xenserver" and currentHost.hypervisorversion == "7.1.0":
+            # Skip tests as volume metrics doesn't see to work
+            self.skipTest("Skipping test because volume metrics doesn't work on hypervisor\
+                            %s, %s" % (currentHost.hypervisor, currentHost.hypervisorversion))
+
+        # Wait for 2 minutes
+        time.sleep(120)
+
+        volume = Volume.list(
+            self.apiclient,
+            virtualmachineid=self.small_virtual_machine.id)[0]
+
+        cmd = listVolumesUsageHistory.listVolumesUsageHistoryCmd()
+        cmd.id = volume.id
+
+        result = self.apiclient.listVolumesUsageHistory(cmd)[0]
+        self.assertEqual(result.id, volume.id)
+        self.assertTrue(hasattr(result, 'stats'))
+        self.assertTrue(type(result.stats) == list and len(result.stats) > 0)
+        stats = result.stats[0]
+        self.assertTrue(hasattr(stats, 'diskioread'))
+        self.assertTrue(hasattr(stats, 'diskiowrite'))
+        self.assertTrue(hasattr(stats, 'diskiopstotal'))
+        self.assertTrue(hasattr(stats, 'diskkbsread'))
+        self.assertTrue(hasattr(stats, 'diskkbswrite'))
+        self.assertTrue(hasattr(stats, 'timestamp'))
+        self.assertTrue(self.valid_date(stats.timestamp))
+
+        return
+
+    def validate_vm_stats(self, stats):
+        self.assertTrue(hasattr(stats, 'cpuused'))
+        self.assertTrue(hasattr(stats, 'diskiopstotal'))
+        self.assertTrue(hasattr(stats, 'diskioread'))
+        self.assertTrue(hasattr(stats, 'diskiowrite'))
+        self.assertTrue(hasattr(stats, 'diskkbsread'))
+        self.assertTrue(hasattr(stats, 'diskkbswrite'))
+        self.assertTrue(hasattr(stats, 'memoryintfreekbs'))
+        self.assertTrue(hasattr(stats, 'memorykbs'))
+        self.assertTrue(hasattr(stats, 'memorytargetkbs'))
+        self.assertTrue(hasattr(stats, 'networkkbsread'))
+        self.assertTrue(hasattr(stats, 'networkkbswrite'))
+        self.assertTrue(hasattr(stats, 'networkread'))
+        self.assertTrue(hasattr(stats, 'networkwrite'))
+        self.assertTrue(hasattr(stats, 'timestamp'))
+        self.assertTrue(self.valid_date(stats.timestamp))
+
     def valid_date(cls, date_text):
         try:
             datetime.datetime.strptime(date_text, '%Y-%m-%dT%H:%M:%S%z')
