@@ -59,7 +59,7 @@ export function login (arg) {
     method: 'post',
     data: params,
     headers: {
-      'Content-Type': 'application/x-www-form-urlencoded'
+      'content-type': 'application/x-www-form-urlencoded'
     }
   })
 }
