@@ -160,6 +160,7 @@ import {
   UserDeleteOutlined,
   UsergroupDeleteOutlined,
   UserOutlined,
+  UserSwitchOutlined,
   UploadOutlined,
   WarningOutlined,
   WifiOutlined,
@@ -313,6 +314,7 @@ export default {
     app.component('UserDeleteOutlined', UserDeleteOutlined)
     app.component('UsergroupDeleteOutlined', UsergroupDeleteOutlined)
     app.component('UserOutlined', UserOutlined)
+    app.component('UserSwitchOutlined', UserSwitchOutlined)
     app.component('UploadOutlined', UploadOutlined)
     app.component('WarningOutlined', WarningOutlined)
     app.component('WifiOutlined', WifiOutlined)
