@@ -20,8 +20,10 @@
 package com.cloud.utils;
 
 import java.io.File;
+import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 
 import org.apache.log4j.Appender;
@@ -29,8 +31,11 @@ import org.apache.log4j.FileAppender;
 import org.apache.log4j.Logger;
 import org.apache.log4j.xml.DOMConfigurator;
 
+import com.google.gson.Gson;
+
 public class LogUtils {
     public static final Logger LOGGER = Logger.getLogger(LogUtils.class);
+    private static final Gson GSON = new Gson();
 
     private static String configFileLocation = null;
 
@@ -72,4 +77,23 @@ public class LogUtils {
         }
         return fileNames;
     }
+
+    public static String logGsonWithoutException(String formatMessage, Object ... objects) {
+        List<String> gsons = new ArrayList<>();
+        for (Object object : objects) {
+            try {
+                gsons.add(GSON.toJson(object));
+            } catch (Exception e) {
+                LOGGER.debug(String.format("Failed to log object [%s] using GSON.", object != null ? object.getClass().getSimpleName() : "null"));
+                gsons.add("error to decode");
+            }
+        }
+        try {
+            return String.format(formatMessage, gsons.toArray());
+        } catch (Exception e) {
+            String errorMsg = String.format("Failed to log objects using GSON due to: [%s].", e.getMessage());
+            LOGGER.error(errorMsg, e);
+            return errorMsg;
+        }
+    }
 }
